require 'exceptions'

class Decide::Acsmi::Contracting::Api::V1::SessionsController < Decide::Acsmi::Contracting::Api::V1::ApiController
  before_action :set_session, only: %i[credentials status]

  def create
    begin
      @session = InitializeSession.call(git_params)

      render json: { session_id: @session.id, embedding_url: sessions_url(session_id: @session.id) }
    rescue ACSmIExceptions::NoSchemaFound, ACSmIExceptions::InvalidSchema
      render json: { session_id: 'none', embedding_url: failed_sessions_url }
    rescue StandardError => e
      return error_response(e.message)
    end
  end

  def failed

  end

  def status
    render json: { status: @session.status }
  end

  private

  def git_params
    result = params.permit(:git_url, :git_username, :git_password)
    result[:git_url] ||= GitAccess::DEFAULT_URL
    return result if result[:git_username].present? && result[:git_password].present?
    result.merge(GitAccess.default_credentials(result[:git_url]))
  end

  def set_session
    @session = Session.find_by_id(params[:id])
    return error_response("Could not find session with ID #{params[:id]}", 404) unless @session
  end

end