class Session < ActiveRecord::Base
  self.primary_key = :id

  belongs_to :user
  has_one :git_access, dependent: :destroy
  has_many :contracting_resources, dependent: :destroy
  has_many :contracts, through: :contracting_resources

  def app_description
    ApplicationDescription.new(self)
  end

  def write_to_repo
    app_description.write(collect_vms_data)
  end

  def status
    contracting_resources_statuses = contracting_resources.pluck(:status).uniq
    return 'in progress' if contracting_resources_statuses.include?('waiting')
    return 'completed' if contracting_resources_statuses.include?('completed')
    'failed' # all contracting tries failed
  end

  def contracting_resource
    contracting_resources.order(index: :asc).select(&:waiting?).first
  end

  def finish!
    remove_git_access
  end

  def remove_git_access
    git_access.try(&:destroy!)
  end

  private

  def collect_vms_data
    endpoint_hash = {
        'cloudbrokerEndpoint' => Setting.value('platform_url'),
        'cloudbrokerUsername' => user.try(&:platform_email),
        'cloudbrokerPassword' => user.try(&:platform_password)
    }

    vms = []
    contracting_resources.each do |c_r|
      if c_r.failed?
        vms << { 'id' => c_r.index.to_s, failed: true }
      elsif c_r.completed_own?
        params = { 'id' => c_r.index.to_s,
                   'own_credentials' => true,
                   'cspName' => c_r.resource_name,
                   'cspId' => c_r.resource_id }
        vms << params.merge(c_r.credential_fields_hash)
      else
        vms << c_r.virtual_machine.description.merge(endpoint_hash)
      end
    end

    {'virtualMachines' => vms}
  end
end