class PrepareResourceAndVM
  require 'platform_connector'

  attr_reader :contracting_resource

  def self.call(contracting_resource)
    new(contracting_resource).perform
  end

  def initialize(contracting_resource)
    @contracting_resource = contracting_resource
  end

  def perform
    # begin
    through_discovery
    # rescue StandartError
    # through_platform
    # end
  end

  private

  def through_discovery
    service = Service.new(contracting_resource.input_resource_id)
    unless service.exists?
      raise "Resource with id #{contracting_resource.input_resource_id} was not found within ACSmI Discovery"
    end

    @contracting_resource.service_id = service.id
    @contracting_resource.service_name = service.class_attribute('servicename')
    mapper_response = ResourceMapper.map(service.attribute('Provider'),
                                         service.attribute('Region'),
                                         service.attribute('Zone'),
                                         service.attribute('Virtual CPU Cores'),
                                         service.attribute('Frequency per Core'),
                                         service.attribute('Memory'),
                                         service.attribute('Instance Storage'),
                                         service.attribute('Optimized for')
    )

    if mapper_response['resource_id']
      @contracting_resource.resource_id = mapper_response['resource_id']
      @contracting_resource.resource_name = PlatformConnector.resource(@contracting_resource.resource_id).try(:[], 'name')
      @contracting_resource.build_virtual_machine(region_id: mapper_response['region_id'],
                                                  instance_type_id: mapper_response['instance_type_id'],
                                                  software_id: '21b7ebed-5076-43b6-8351-0e06cf16eedc',
                                                  image: 'Ubuntu 16.04')
    else
      # temp stub, use default value in case anything goes wrong
      @contracting_resource.use_default_data
    end

    @contracting_resource.save!
  end

  def through_platform
    raise 'deprecated. should be updated to deployment_id one day'
    resource = PlatformConnector.resource(contracting_resource.input_resource_id)
    raise "Resource with id #{contracting_resource.input_resource_id} wan not found within CloudBroker Platform"
    contracting_resource.update!(resource_id: resource['id'], resource_name: resource['name'])
  end

end