/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.repository;

import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import eu.decideh2020.acsmi.backend.services.server.domain.Service;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceDevOpsQueryInfoVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceFullInfoVO;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceNFRInfoExtendedVO;


/**
 * Spring Data JPA repository for the Service entity.
 */
@Repository
public interface ServiceRepository extends JpaRepository<Service,Long>, ServiceRepositoryCustom {
	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.Service(s.id, s.servicename, s.serviceclassid, sc.serviceclassname, s.deleteddate, si.id, si.timestamp, si.detail, si.createdDate, si.lastModifiedDate) from Service s join ServiceClass sc on sc.id = s.serviceclassid left outer join ServiceIncidence si on s.id = si.serviceid order by s.servicename")
	public List<Service> findAllServices();
	
	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceFullInfoVO(s.id, s.servicename, s.firstcontract, s.secondcontract, s.thirdcontract, sc.id, sc.serviceclassname, sat.id, sat.name, sat.nfrname, sat.isenumeration, sat.ischecklist, sat.iscommon, sat.isfunctionalrequirement, sat.parentid, sat.childid, sat.units, sat.unitfactor, sat.unitrule, sat.checklistevalrule, sav.unitvalue, sav.checklistvalue, sav.id, sav.serviceattributevalue, d.value, s.deleteddate) from Service s join ServiceClass sc on s.serviceclassid = sc.id join ServiceAttributeValue sav on s.id = sav.serviceid join ServiceAttributeType sat on sav.serviceattributetypeid = sat.id left outer join Definition d on sav.serviceattributevalue = d.code where s.id = ?1 order by sat.id")
	public List<ServiceFullInfoVO> findService(Long id);

	@Query("select concat(sc.serviceclassname, ';', d.value) as devopskey FROM ServiceClass sc, Definition d where d.typeid = 3")
	public List<String> getDevOpsKeys();

	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceDevOpsQueryInfoVO(concat(sc.serviceclassname, ';', d.value), s.servicename) from Service s join ServiceClass sc on s.serviceclassid = sc.id join ServiceAttributeValue sav on s.id = sav.serviceid join Definition d on sav.serviceattributevalue = d.code where d.typeid = 3 order by concat(sc.serviceclassname, ';', d.value), s.servicename")
	public List<ServiceDevOpsQueryInfoVO> getDevOpsQueryInfo();
	
	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.vo.ServiceNFRInfoExtendedVO(s.id, s.servicename, sc.serviceclassname, sat.name, sat.nfrname, sav.serviceattributevalue, sat.isenumeration, sat.iscommon) from ServiceAttributeType sat join ServiceAttributeValue sav on sat.id = sav.serviceattributetypeid join Service s on sav.serviceid = s.id join ServiceClass sc on s.serviceclassid = sc.id where sat.isfunctionalrequirement = false and s.id in (?1)")
	public List<ServiceNFRInfoExtendedVO> getNFRInfo(List<Long> serviceids);
	
	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.Service(s.id, s.servicename, s.serviceclassid, sc.serviceclassname, s.deleteddate, si.id, si.timestamp, si.detail, si.createdDate, si.lastModifiedDate) from Service s join ServiceClass sc on sc.id = s.serviceclassid left outer join ServiceIncidence si on s.id = si.serviceid where s.id in (?1)")
	public List<Service> getOptimusInfo(List<Long> serviceids);
	
	@Query("select s from Service s where s.servicename = ?1")
	public Service checkIfServiceExistsForInsert(String name);
	
	@Query("select s from Service s where s.servicename = ?2 and s.id <> ?1")
	public Service checkIfServiceExistsForUpdate(Long identifier, String name);
	
	@Query("select new eu.decideh2020.acsmi.backend.services.server.domain.Service(s.id, s.servicename, s.serviceclassid, sc.serviceclassname, s.deleteddate, si.id, si.timestamp, si.detail, si.createdDate, si.lastModifiedDate) from Service s join ServiceClass sc on sc.id = s.serviceclassid join ServiceAttributeValue sav on s.id = sav.serviceid join Definition d on sav.serviceattributevalue = d.code left outer join ServiceIncidence si on s.id = si.serviceid where d.typeid = ?1 and d.value = ?2 order by s.servicename")
	public List<Service> findServicesByCSP(Long typeid, String cspname);
}
