/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.service;

import java.io.InputStream;

import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;
import org.springframework.web.util.UriComponents;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;
import eu.decideh2020.acsmi.backend.services.server.domain.vo.InfluxDBConfigVO;
import eu.decideh2020.acsmi.backend.services.server.repository.DatasourceConfig;
import eu.decideh2020.acsmi.backend.services.server.web.rest.vm.LoginVM;

@Service
public class RequestToken {
	
	private final Logger log = LoggerFactory.getLogger(RequestToken.class);
	
	private String getToken(String environmentFilename, UriComponents uriBuilder) {
		log.debug("getToken  ::  create mapper");
		ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
		log.debug("getToken  ::  mapper created");

        try {
        	log.debug("getToken  ::  environmentFilename = " + environmentFilename);
        	log.debug("getToken  ::  uriBuilder = " + uriBuilder);
        	log.debug("getToken  ::  get credentials");
        	InputStream inputStream = null;
    		inputStream = this.getClass().getClassLoader().getResourceAsStream("config/" + environmentFilename);
    		LoginVM credentials = mapper.readValue(inputStream, LoginVM.class);   		
        	//LoginVM credentials = mapper.readValue(new File("src\\main\\resources\\config\\" + environmentFilename), LoginVM.class);
        	log.debug("getToken  ::  get credentials end");
            
            log.debug("SOLICITAMOS EL TOKEN PARA COMUNICARNOS CON EL REST");
            final String uriToken;
            
            if(uriBuilder == null) {
            	
            	UriComponents generatedUriBuilder = ServletUriComponentsBuilder.fromCurrentRequestUri().build();
            	
            	uriToken = RequestURI.getURI(generatedUriBuilder, "/api/authenticate");
            	
            } else {
            	
            	uriToken = RequestURI.getURI(uriBuilder, "/api/authenticate");
            	
            }
            
            RestTemplate restTemplateToken = new RestTemplate();

			HttpEntity<LoginVM> requestToken = new HttpEntity<LoginVM>(credentials, null);
			
			ResponseEntity<String> responseToken = restTemplateToken.postForEntity(uriToken, requestToken, String.class);
			
			JSONObject responseBody = new JSONObject(responseToken.getBody());
			
			String token = responseBody.getString("id_token");
			
			return token;
            

        } catch (Exception e) {
        	log.error("getToken ::  Exception  ::  " + e.getMessage());
            return null;
        }
	}
	
	private InfluxDBConfigVO getInfluxDBConfig(String environmentFilename){
		try {
			ObjectMapper mapper = new ObjectMapper(new YAMLFactory());
        	log.debug("getToken  ::  environmentFilename = " + environmentFilename);
        	log.debug("getToken  ::  get InfluxDBConfig"); 
        	InputStream inputStream = null;
    		inputStream = this.getClass().getClassLoader().getResourceAsStream("config/" + environmentFilename);
    		InfluxDBConfigVO influxdbconfig = mapper.readValue(inputStream, InfluxDBConfigVO.class);   		
			//InfluxDBConfigVO influxdbconfig = mapper.readValue(new File("src\\main\\resources\\config\\" + environmentFilename), InfluxDBConfigVO.class);
			log.debug("getToken  ::  get InfluxDBConfig end");
			return influxdbconfig;
        } catch (Exception e) {
        	e.printStackTrace();
            return null;
        }
	} 
	
	@Component
	@Profile("dev")
	public class DevDatasourceConfig implements DatasourceConfig {
	    @Override
	    public String setup(UriComponents uriBuilder) {
	    	log.debug("setup  ::  Setting up datasource for DEV environment... ");	        
	        return getToken("credentials-dev.yml", uriBuilder);
	    }
	    
	    @Override
	    public InfluxDBConfigVO setupInfluxDB() {
	    	log.debug("   **********  DEV  **********");
	    	log.debug("setupInfluxDB  ::  Setting up datasource for DEV environment... ");
	    	return getInfluxDBConfig("influxdb-dev.yml");
	    }
	}
	
	@Component
	@Profile("prod")
	public class ProductionDatasourceConfig implements DatasourceConfig {
		@Override
	    public String setup(UriComponents uriBuilder) {
	    	log.debug("setup  ::  Setting up datasource for PRODUCTION environment... ");
	    	return getToken("credentials-prod.yml", uriBuilder);
	    }
		
		@Override
	    public InfluxDBConfigVO setupInfluxDB() {
			log.debug("   **********  PROD  **********");
	    	log.debug("setupInfluxDB  ::  Setting up datasource for PRODUCTION environment... ");
	    	return getInfluxDBConfig("influxdb-prod.yml");
	    }
	}
}
