/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
package eu.decideh2020.acsmi.backend.services.server.web.rest;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.codahale.metrics.annotation.Timed;

import eu.decideh2020.acsmi.backend.services.server.domain.Definition;
import eu.decideh2020.acsmi.backend.services.server.repository.DefinitionRepository;
import eu.decideh2020.acsmi.backend.services.server.web.rest.util.HeaderUtil;
import io.github.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing Definition.
 */
@RestController
@RequestMapping("/api")
public class DefinitionResource {

    private final Logger log = LoggerFactory.getLogger(DefinitionResource.class);

    private static final String ENTITY_NAME = "definition";
        
    private final DefinitionRepository definitionRepository;

    public DefinitionResource(DefinitionRepository definitionRepository) {
        this.definitionRepository = definitionRepository;
    }


    /**
     * GET  /definitions : get all the definitions.
     *
     * @return the ResponseEntity with status 200 (OK) and the list of definitions in body
     */
    @GetMapping("/definitions")
    @Timed
    @Transactional
    public ResponseEntity<List<Definition>> getAllDefinitions() {
       log.debug("REST request to get all Definitions");
        try {log.debug("getAllDefinitions  ::  getting all definitions...");
        	return ResponseEntity.ok(definitionRepository.findAll());
        } catch (Exception e) {
        	log.error("getAllDefinitions  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(
                    		ENTITY_NAME, "definitionGetAllError", "Cannot get the list of Definitions")).body(null);
        }
    }

    
    /**
     * GET  /definitions/:id : get the "id" definition.
     *
     * @param id the id of the definition to retrieve
     * @return the ResponseEntity with status 200 (OK) and with body the definition, or with status 404 (Not Found)
     */
    @GetMapping("/definitions/{id}")
    @Timed
    public ResponseEntity<Definition> getDefinition(@PathVariable Long id) {
        log.debug("REST request to get Definition : {}", id);
        Definition definition = null;
        try {
        	log.debug("getDefinition  ::  getting definition...");
        	definition = definitionRepository.findOne(id);
        	
        	log.debug("getDefinition  ::  getting OK");
            return ResponseUtil.wrapOrNotFound(Optional.ofNullable(definition));
        } catch (Exception e) {
        	log.error("getDefinition  ::  Exception  ::  " + e.getMessage());
        	return ResponseEntity.badRequest()
                    .headers(HeaderUtil.createFailureAlert(ENTITY_NAME, "definitionGetError", "Cannot get the Definition"))
                    .body(null);
        }
    }
    
    
    /**
     * Converts a definition list in a Map
     * @param list The original definition list
     * @return Map
     */
    public static Map<String, String> getDefinitionMap(final List<Definition> list) {
    	Map<String, String> map = null;
    	if (list != null && !list.isEmpty()) {
    		map = new HashMap<String, String>();
    		for (Definition def : list) {
    			map.put(def.getCode(), def.getValue());
    		}
    	}
    	return map;
    }
}
