/*******************************************************************************
 * Copyright (c) 2017 Tecnalia.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 	
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * The code has been contributed over a stub generated with the JHipster
 * Application generator http://www.jhipster.tech/.
 * 
 * Contributors (in alphabetical order):
 * 
 * Gorka Benguria                       Tecnalia
 * Iñaki Etxaniz                        Tecnalia
 * Juncal Alonso                        Tecnalia
 * Leire Orue-Echevarria                Tecnalia
 * Maria Jose Lopez                     Tecnalia
 * Marisa Escalante                     Tecnalia
 * Alberto Molinuevo					Tecnalia
 * 
 * Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
 ******************************************************************************/
(function() {
    'use strict';
    
    angular
        .module('acsmiApp')
        .controller('ServiceDialogController', ServiceDialogController);

    ServiceDialogController.$inject = ['$timeout', '$scope', '$uibModalInstance', 'entity', 'Service', 'ServiceClass', 'Definition', 'Principal'];

    function ServiceDialogController($timeout, $scope, $uibModalInstance, entity, Service, ServiceClass, Definition, Principal) {
    	
    	var vm = this;

        vm.service = entity;
        vm.clear = clear;
        vm.save = save;
        
        vm.selectedclassid = null;
        vm.selectedclass = null;
        
        vm.endorseFRInvalidValue = null;
        vm.endorseFRInvalidValueField = null;
        vm.endorseNFRInvalidValue = null;
        vm.endorseNFRInvalidValueField = null;
        
        vm.uploadError = false;
        
        vm.user = null;
        
        loadServiceClasses();
        
        
        // Carga la lista de clases de servicio
        function loadServiceClasses() {
        	Principal.identity().then(function(account) {
        		vm.user = account;
        		ServiceClass.fullquery({},
    	        		function success(result) {
    	        			vm.serviceClasses = result;
    	        			loadDefinitions();
    	        		},
    	        		function error(error) {}
    	        );        		
            });
        }
        
        
        // Carga la lista de definiciones
        function loadDefinitions() {
	        Definition.query({},
	        		function success(result) {
						vm.definitions = result;
						if (vm.service.id != null && vm.service.id != 0) {
							for (var i = 0;i < vm.serviceClasses.length;i++) {
								if (vm.serviceClasses[i].id == vm.service.serviceclassid) {
									vm.selectedclassid = vm.serviceClasses[i].id;
									vm.selectedclass = vm.serviceClasses[i];
									break;
								}
							}
							$scope.reindexAttributes();
	        				$scope.loadAttributes();
	        				$scope.loadChildren(null, true, true, true);
	        			}
					},
					function error(error) {}
	        );
        }        
        
        // Reindexa los atributos de un servicio
        $scope.reindexAttributes = function() {
        	var attributes = new Array();
        	for (var i = 0;i < vm.service.attributes.length;i++) {
        		attributes[vm.service.attributes[i].typeid] = vm.service.attributes[i];
        	}
        	vm.service.attributes = attributes;
        }
        
        // Carga la lista de atributos hijos
        $scope.loadChildrenByType = function(definition, array, callingattrid) {
        	for (var j = 0;j < array.length;j++) {
        		// Solo se cargan los hijos de los atributos que son padre
        		if (array[j].parentid != null) {
	        		// Comprobar la carga actual del array de hijos. Si el array de hijos es nulo/vacío, reinicializar
					if (vm.children[array[j].typeid] == null || vm.children[array[j].typeid].length == 0) {
						vm.children[array[j].typeid] = new Array();
					}
					
	        		if (callingattrid == null) {	        			
	        			// Si la llamada no la hace ningún atributo, insertar definiciones
	        			if (array[j].typeid == definition.typeid
	        					&& definition.code.endsWith(vm.service.attributes[array[j].parentid].value.substring(2))) {
		        			vm.children[array[j].typeid].push( {code: definition.code, value: definition.value, typeid: definition.typeid });
	        			}
	        		} else if (array[j].parentid == callingattrid
	        				&& array[j].typeid == definition.typeid
							&& definition.code.endsWith(vm.service.attributes[array[j].parentid].value.substring(2))) {
						// Si la llamada la realiza un atributo, buscar su atributo hijo, y a su vez buscar las definiciones que le corresponden
	        			// A su vez, si el array de hijos contiene definiciones diferentes a las que se necesitan, reinicializar
						if (vm.children[array[j].typeid].length > 0
								&& vm.children[array[j].typeid][0].code.substring(2) != definition.code.substring(2)) {
							vm.children[array[j].typeid] = new Array();
						}
								
						// Insertar definición en el array de hijos
						vm.children[array[j].typeid].push({code: definition.code, value: definition.value, typeid: definition.typeid });
						vm.service.attributes[array[j].typeid].value = null;
	        		}
				}
        	}
        }
        
        
        // Carga el array de atributos checklist
        $scope.loadChecklistArray = function() {
        	for (var i = 0;i < vm.definitions.length;i++) {
        		if (vm.definitions[i].ischeckoption) {
        			// Inicializar array de checklists
        			if (vm.checklists[vm.definitions[i].typeid] == null) {
        				vm.checklists[vm.definitions[i].typeid] = new Array();
					}
        			
        			// Insertar valores en el array general
        			vm.checklists[vm.definitions[i].typeid].push({code: vm.definitions[i].code,
        				value: vm.definitions[i].value, typeid: vm.definitions[i].typeid });
        			
        			// Asignar valores del atributo
        			if (vm.service.attributes[vm.definitions[i].typeid] != null) {
        				if (vm.service.attributes[vm.definitions[i].typeid].arraychecklist == null) {
        					vm.service.attributes[vm.definitions[i].typeid].arraychecklist = new Array();
        				}
            			if (vm.service.id != null && vm.service.attributes[vm.definitions[i].typeid].checklistvalue != null) {
            				vm.service.attributes[vm.definitions[i].typeid].arraychecklist =
            					convertStringToArray(vm.service.attributes[vm.definitions[i].typeid].checklistvalue, ",");
               			}
        			}
        		}
        	}
        }
       	
        
       	// Prepara la carga de la lista de atributos hijos y checklists
   		$scope.loadChildren = function(callingattrid, iscommon, isfr, isnonfr) {
   			// Cargar atributos hijos
   			for (var i = 0;i < vm.definitions.length;i++) {
   				if (!vm.definitions[i].ischeckoption) {
   					// Cargar hijos de atributos comunes
   	   				if (iscommon) {
   	  					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.commonattributes, callingattrid);
   	   				}
   	   				
   	   				// Cargar hijos de atributos funcionales
   	   				if (isfr) {
   	   					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.functionalattributes, callingattrid);
   	   				}   				
   	   				
   	   				// Cargar hijos de atributos no funcionales
   	   				if (isnonfr) {
   	   					$scope.loadChildrenByType(vm.definitions[i], vm.selectedclass.nonfunctionalattributes, callingattrid);
   	   				}
   				}
   			}
   		}
   		
   		
   		// Configura las unidades de un atributo
   		$scope.configUnits = function(attribute) {
   			attribute.arrayunits = new Array();
   			if (attribute.units != null) {
   				attribute.arrayunits = convertStringToArray(attribute.units, ",");
   			}
   			return attribute;
   		}
   		
   		
   		// Carga un array de la lista de atributos
   		$scope.loadAttributesArray = function(array) {
   			for (var i = 0;i < array.length;i++) {
        		if (vm.service.id == null) {
	        		vm.service.attributes[array[i].typeid] = array[i];
	        		vm.service.attributes[array[i].typeid].value = null;
	        		vm.service.attributes[array[i].typeid] =
	        			$scope.configUnits(vm.service.attributes[array[i].typeid]);
        		} else {
        			array[i] = $scope.configUnits(array[i]);
        		}
        	}
   		}

        
        // Carga la lista de atributos
        $scope.loadAttributes = function() {
        	for (var i = 0;i < vm.serviceClasses.length;i++) {
        		if (vm.serviceClasses[i].id == vm.selectedclassid) {
        			vm.selectedclass = vm.serviceClasses[i];
        			break;
        		}
        	}
        	
        	// Inicializar arrays
        	vm.children = new Array(new Array());
        	vm.checklists = new Array(new Array());
        	
        	// Cargar variables
        	if (vm.service.id == null) {
        		vm.service.serviceclassid = vm.selectedclass.id;
        		vm.service.serviceclassname = vm.selectedclass.serviceclassname;
        		
            	vm.service.attributes = new Array();
        	}
        	
        	// Cargar array de checklists
        	$scope.loadChecklistArray();

        	// Cargar arrays de atributos
        	$scope.loadAttributesArray(vm.selectedclass.commonattributes);
        	$scope.loadAttributesArray(vm.selectedclass.functionalattributes);
        	$scope.loadAttributesArray(vm.selectedclass.nonfunctionalattributes);
        	
        	// Resolver información de CSP
        	if (vm.user.userType == 'csp' && vm.service.id == null) {
        		for (var i = 0;i < vm.definitions.length;i++) {
        			if (vm.definitions[i].value == vm.user.company) {
        				vm.service.attributes[3].value = vm.definitions[i].code;
        				break;
        			}
        		}
        	}
        }
        
        
        // Realiza el tratamiento de chequeado de una opción en una checklist
        $scope.checkOption = function(typeid, value) {
        	if (vm.service.attributes[typeid].arraychecklist == null) {
        		vm.service.attributes[typeid].arraychecklist = new Array();
        		vm.service.attributes[typeid].arraychecklist.push(value);
        	} else {
        		var index = vm.service.attributes[typeid].arraychecklist.indexOf(value);
        		if (index == -1) {
                	vm.service.attributes[typeid].arraychecklist.push(value);
                } else {
                	vm.service.attributes[typeid].arraychecklist.splice(index, 1);
                }
        	}
        	
        	// En el alta, es necesario asignar un valor a la propiedad "value" para validar el formulario
        	if (vm.service.id == null) {
        		vm.service.attributes[typeid].value = value;
        	}
        }
        
        
        // Valida el formato de un valor de un campo
        $scope.validateEndorseInput = function(units, typeid, isFR) {
        	if (units != null && units.length > 0 && vm.service.attributes[typeid].value != null && vm.service.attributes[typeid].value.length > 0) {
        		vm.service.attributes[typeid].value = vm.service.attributes[typeid].value.replace(",", ".");
        		if (isFR) {
        			vm.endorseFRInvalidValue = null;
        			vm.endorseFRInvalidValueField = null;
            		if (isNaN(vm.service.attributes[typeid].value)) {
            			vm.endorseFRInvalidValueField = vm.service.attributes[typeid].name;
            			vm.endorseFRInvalidValue = vm.service.attributes[typeid].value;
            			vm.service.attributes[typeid].value = null;
            		}
        		} else {
        			vm.endorseNFRInvalidValue = null;
        			vm.endorseNFRInvalidValueField = null;
            		if (isNaN(vm.service.attributes[typeid].value)) {
            			vm.endorseNFRInvalidValueField = vm.service.attributes[typeid].name;
            			vm.endorseNFRInvalidValue = vm.service.attributes[typeid].value;
            			vm.service.attributes[typeid].value = null;
            		}
        		}
        	} else {
        		vm.endorseFRInvalidValue = null;
    			vm.endorseFRInvalidValueField = null;
    			vm.endorseNFRInvalidValue = null;
    			vm.endorseNFRInvalidValueField = null;
        	}
        }
        
        
        // Subida de documentos 
        $scope.addFile = function(elem, index) {
        	var allowedExtensions = ['PDF'];
        	
        	if (elem.files.length > 0) {
        		// Comprobar extensiones permitidas
        		var extensionOk = false;
	        	var extension = elem.files[0].name.split(".").pop().toUpperCase();
	        	for (var i = 0;i < allowedExtensions.length;i++) {
	        		if (extension == allowedExtensions[i]) {
	        			extensionOk = true;
	        			break;
	        		}
	        	}

        		// Validaciones
        		if (!extensionOk) {
        			vm.uploadError = true;
        			angular.element(index).val(null);
        			if (index == 0) {
        				vm.service.firstcontract = null;
        			} else if (index == 1) {
        				vm.service.secondcontract = null;
        			} else if (index == 2) {
        				vm.service.thirdcontract = null;
        			}
        			$scope.$apply();
        		} else {
        			// Si pasa todas las validaciones, subir fichero
        			vm.uploadError = false;
        			var reader = new FileReader();
            		reader.onload = function(e) {            			
            			// Convertir a array de bytes
            			if (index == 0) {
            				vm.service.firstcontract = btoa(e.target.result.toString());
            			} else if (index == 1) {
            				vm.service.secondcontract = btoa(e.target.result.toString());
            			} else if (index == 2) {
            				vm.service.thirdcontract = btoa(e.target.result.toString());
            			}
            			//$scope.$apply();
            			$scope.$digest();
            		};
            		reader.readAsBinaryString(elem.files[0]);
        		}
        	} else {
        		vm.uploadError = true;
        		$scope.removeFile(index);
        	}
        };
        
        
        // Borrado de documentos
        $scope.removeFile = function(index) {
        	if (index == 0) {
        		vm.service.firstcontract = null;
        	} else if (index == 1) {
        		vm.service.secondcontract = null;
        	} else if (index == 2) {
        		vm.service.thirdcontract = null;
        	}
        	$scope.$apply();
        }
        
        
        // Decodifica el fichero PDF de base de datos
        $scope.decode = function(data) {
        	var base64str = data;
			var binary = atob(base64str.replace(/\s/g, ''));
			var len = binary.length;
			var buffer = new ArrayBuffer(len);
			var view = new Uint8Array(buffer);
			for (var i = 0;i < len;i++) {
			    view[i] = binary.charCodeAt(i);
			}
			return view;
        }
        
        
        // Descarga de documentos
        $scope.download = function(index) {
        	var urlCreator = window.URL || window.webkitURL || window.mozURL || window.msURL;
    		if (urlCreator) {
    			var hiddenElement = document.createElement("a");
    			var blob = null;
    			if (index == 0) {
    				blob = new Blob( [$scope.decode(vm.service.firstcontract)], { type: "application/pdf" });
    	        	hiddenElement.download = vm.service.servicename + "_Overarching_services_contract.pdf";
            	} else if (index == 1) {
            		blob = new Blob( [$scope.decode(vm.service.secondcontract)], { type: "application/pdf" });
            		hiddenElement.download = vm.service.servicename + "_SLA_contract.pdf";
            	} else if (index == 2) {
            		blob = new Blob( [$scope.decode(vm.service.thirdcontract)], { type: "application/pdf" });
            		hiddenElement.download = vm.service.servicename + "_Data_processing_contract.pdf";
            	}
    			var url = urlCreator.createObjectURL(blob);
    			hiddenElement.href = url;
    			hiddenElement.target = "_blank";
    	        hiddenElement.style = "display: none";
    	        document.body.appendChild(hiddenElement);
    	        hiddenElement.click();
    	        
    	        setTimeout(function() {window.URL.revokeObjectURL(url), 100});
    		}
        }
        
        
        // Convierte un String en un array
        function convertStringToArray(string, separator) {
        	if (string != null) {
        		return string.split(separator);
        	}
        	return new Array();
        }
        
        
        // Convierte un array en un String
        function convertArrayToString(array, separator) {
        	if (array != null && array.length > 0) {
        		var string = "";
        		for (var i = 0;i < array.length;i++) {
        			string += array[i];
        			if (i < array.length - 1) {
        				string += separator;
        			}
        		}
        		return string;
        	}
        	return null;
        }

        
        // Inserta o actualiza el servicio
        function save() {
        	// Transformar atributos con unidades y checklists
        	for (var i = 0;i < vm.service.attributes.length;i++) {
        		if (vm.service.attributes[i] != null) {
        			// Eliminar array auxiliar de unidades
        			if (vm.service.id == null) {
        				delete vm.service.attributes[i].arrayunits;
        			}
        			
        			// Evaluar array auxiliar de checklists
        			if (vm.service.attributes[i].ischecklist) {
        				vm.service.attributes[i].checklistvalue =
        					convertArrayToString(vm.service.attributes[i].arraychecklist, ",");
            			
            			// Aplicar regla
            			var variable = vm.service.attributes[i].arraychecklist.length;
            			vm.service.attributes[i].value = eval(vm.service.attributes[i].checklistevalrule);
            			delete vm.service.attributes[i].arraychecklist;
            		}
        		}
        	}
        	
        	// Transformar atributo "Region"
        	if (vm.service.attributes[2].value != null && vm.service.attributes[2].value.length == 0) {
        		vm.service.attributes[2].value = null;
        	}
        	
        	// Guardar
            if (vm.service.id != null) {
            	Service.update({attributes: vm.service.attributes}, vm.service, onSaveServiceSuccess, onSaveServiceError);
            } else {
            	Service.save({attributes: vm.service.attributes}, vm.service, onSaveServiceSuccess, onSaveServiceError);
            }
        }

        
        // Función de éxito en alta/modificación de servicio
        function onSaveServiceSuccess(result) {
            $scope.$emit('acsmiApp:serviceUpdate', result);
            $uibModalInstance.close(result);
        }

        
        // Función de error en alta/modificación de servicio
        function onSaveServiceError() {}
        
        
        $timeout(function() {
            angular.element('.form-group:eq(1)>input').focus();
        });

        
        function clear() {
            $uibModalInstance.dismiss('cancel');
        }
    }
})();
